#include "MainWindow.h"
#include "ui_MainWindow.h"
#include "exr.h"
#include <fstream>
#include <cstdlib>
#include <algorithm>
#include <QFileDialog>
#include <QPixmap>
#include <QImage>
#include <QProgressDialog>

//----------------------------------------------------------------------------------------------------------------------
MainWindow::MainWindow(
                       QWidget *_parent
                      ) :
                         QMainWindow(_parent),
                         m_ui(new Ui::MainWindow)
{
  m_ui->setupUi(this);
  connect(m_ui->m_loadEXR,SIGNAL(clicked()),this,SLOT(LoadEXR()));
  connect(m_ui->m_cut,SIGNAL(clicked()),this,SLOT(DoMediunCut()));

}

//----------------------------------------------------------------------------------------------------------------------
void MainWindow::DoMediunCut()
{
  if(m_exrName.isEmpty())
      return;

  char tmp[100];
  QProgressDialog progress("Doing Median Cut", "Abort ", 0, m_ui->m_numLights->value()+2, this);
  progress.setWindowModality(Qt::WindowModal);
  progress.setMinimumDuration(0);
  progress.setValue(1);

  // Load the image
  bool r;
  MedianCut cut( m_exrName.toAscii() , r );
  if ( !r )
  {
    std::cerr<<": Couldn't load this image." <<std::endl;
    return ;
  }
  std::vector<MCRegion> regions;

  try
  {
    // Perform the cut
    cut.cut( m_ui->m_numLights->value() , regions );
    // Create the lights
    cut.createLightsFromRegions( regions );

   }
   catch( Error& e )
   {
    std::cerr<<"Error thrown: " << e.error ;
    return;
    }
    progress.setValue(2);

    float theta = 0, phi = 0;
    float minLum = 9999, maxLum = -9999;
    std::vector<ShadowPointIndex> shadowPointList;
    size_t currentShadowLight = 0;
    size_t shadowLightStart = 0;
    int shadowCount = 0;
    // The number of desired shadow lights
    int numShadowLights=-1;
    // Ignore the lights
    if ( numShadowLights != 0 )
    {
      // Count the number of lights
      for ( int i = 0; i < m_ui->m_numLights->value(); ++i )
      {
        // Get the light
        const Point& point = cut.getLight( i );

        // Check the intensity
        if ( point.lum() >=m_ui->m_shadowThreshold->value() )
        {
          shadowCount++;
          shadowPointList.push_back( ShadowPointIndex( i, point ) );
        }
        progress.setValue(i);

        if (progress.wasCanceled())
            break;
      }

      // Print some info
      sprintf( tmp, "There are %d basic bright lights (before trimming to %d).", (int)shadowPointList.size(), (int)numShadowLights );
      std::cerr<< tmp <<std::endl;

      // Sort the shadow point list
      std::sort( shadowPointList.begin(), shadowPointList.end(), ShadowPointSorter() );

      // Now get the shadow light threshold
      if ( numShadowLights != -1 )
      {
        // Get the light cutoff
        shadowLightStart = shadowPointList.size() - numShadowLights;

        sprintf( tmp, "Starting shadows at light #%d.", (int)shadowLightStart );
        std::cerr<< tmp <<std::endl;
      }
    }

    // Get the normalisation value
    float normalise= 1;// / (float)numLights;
    if (m_ui->m_normalize->isChecked() == true)
      normalise=1/(float)m_ui->m_numLights->value();
    // The brightest light transform
    float brightest = 0;
    std::string oFile=m_exrName.toStdString();
    oFile.erase(oFile.find("."));
    std::string lightmap=oFile+".lightmap";
    std::fstream FileOut;
    FileOut.open(lightmap.c_str(),std::ios::out|std::ios::binary);
    if (!FileOut.is_open())
    {
      std::cout <<"Could not open File : "<<m_exrName.toStdString()<<" for writing "<<std::endl;
      return;
    }

    std::fstream RibOut;

    if(m_ui->m_ribOut->isChecked() == true)
      {
      std::string RibFile=oFile+".rib";
      RibOut.open(RibFile.c_str(),std::ios::out|std::ios::binary);
      if (!RibOut.is_open())
        {
        std::cout <<"Could not open File : "<<m_exrName.toStdString()<<".rib for writing "<<std::endl;
         //exit(1);
        }
      }

    // For each point marked on the image, create a transform and a light...
    for ( int i = 0; i < m_ui->m_numLights->value(); ++i )
    {
      // Get the light
      const Point& point = cut.getLight( i );

      // Convert the point position to spherical coordinates
      point.angles( theta, phi,m_ui->m_lightLayout->currentIndex());

      // Get the luminance
      float lum = point.lum();
      if ( lum < minLum ) minLum = lum;
      if ( lum > maxLum ) maxLum = lum;

      // Is it the brightest?
      if ( lum > brightest )
      {
        // Remember this
        brightest = lum;
      //	brightest = M;
      }

      // Is it shadowed?
      bool shadowed = lum > m_ui->m_shadowThreshold->value();

      // Check if it if *really* is shadowed
      if ( shadowed )
      {
        // Check if it's in range
        if ( currentShadowLight < shadowLightStart )
        {
          // Turn shadows off
          shadowed = false;
        }

        // Increment the current shadow light
        currentShadowLight++;
      }
      


       // OUTPUT FORMAT
       // x y z r g b intensity Shadowed  shadow angle shadow penumbra

        // print out
        //normalise=1.0;
        float radius=m_ui->m_Radius->value();
        float intensity=m_ui->m_intensity->value();
        // The angle of the shadow map
        float shadowAngle=m_ui->m_shadowAngle->value();
        // The punumbra
        float shadowPenumbra=m_ui->m_shadowPenumbra->value();

        FileOut << (radius*cos(phi))*cos(theta)<<" ";
        FileOut << radius*sin(phi)<<" ";
        FileOut << (radius*cos(phi))*sin(theta)<<" ";

        FileOut <<std::min(point.m_colour.r * normalise,1.0f)
                <<" "<< std::min(point.m_colour.g * normalise,1.0f)
                <<" "<< std::min( point.m_colour.b * normalise,1.0f)<<" ";
        FileOut << ((shadowed == true) ? "1" : "0");
        FileOut <<" "<<intensity<<" ";
        FileOut <<" "<<shadowAngle<<" " <<shadowPenumbra;

        FileOut<<std::endl;

        //# LightSource "pointlight" 1 "from" [-2 2 4] "intensity" [10] "lightcolor" [ r g b ]
        if (m_ui->m_ribOut->isChecked() == true)
        {
          RibOut <<"Declare \"Light"<<i<<"\" \"string\"\n";

          RibOut <<"TransformBegin \n";
          RibOut <<"\tTranslate "<<(radius*cos(phi))*cos(theta)<<" ";
          RibOut <<-(radius*sin(phi))<<" "<<(radius*cos(phi))*sin(theta)<<"\n ";

          if (!shadowed)
          {
            RibOut <<"\tLightSource \"pointlight\" \"Light"<< i <<  "\" \"intensity\" ["<<intensity<<"] ";
            RibOut <<"\"lightcolor\" ["<<point.m_colour.r * normalise<<" "<< point.m_colour.g * normalise<<" "<<  point.m_colour.b * normalise<<"]\n ";
          }
          else
          {
            RibOut <<"\tLightSource \"shadowpoint\" \"Light"<< i <<  "\" \"intensity\" ["<<intensity<<"] ";
            RibOut <<"\"lightcolor\" ["<<point.m_colour.r * normalise<<" "<< point.m_colour.g * normalise<<" "<<  point.m_colour.b * normalise<<"]\n";
            RibOut<<" \"string sfpx\"  [\"raytrace\"] ";
            RibOut<<" \"string sfnx\"  [\"raytrace\"] ";
            RibOut<<" \"string sfpy\"  [\"raytrace\"] ";
            RibOut<<" \"string sfny\"  [\"raytrace\"] ";
            RibOut<<" \"string sfpz\"  [\"raytrace\"] ";
            RibOut<<" \"string sfnz\"  [\"raytrace\"]\n";
            }
          RibOut <<"TransformEnd\n";

          }
    }


    // Try and write out a map
    try
    {
      // Write out the regions
      cut.writeRegionMap( oFile.c_str(), regions );
      EXR exrImage;
      oFile+=".map.exr";
      bool exrLoaded=exrImage.load(oFile.c_str());
      std::cout<<oFile<<std::endl;

      if(exrLoaded == true)
      {
        int Width=exrImage.width();
        int Height=exrImage.height();

        // now load the exr pixels to the byte array
        Imf::Rgba Pixel;
        QImage image(Width, Height, QImage::Format_RGB16);
        QColor c;
        for(int y=0; y<Height; ++y)
        {
          for(int x=0; x<Width; ++x)
          {
            Pixel=exrImage.pixel(x,y);
            c.setRedF(std::min(float(Pixel.r),1.0f));
            c.setGreenF(std::min(float(Pixel.g),1.0f));
            c.setBlueF(std::min(float(Pixel.b),1.0f));
            image.setPixel(x, y, c.rgb());
          }
        }
        m_ui->m_exrCutImage->setScaledContents(true);
        QImage toFit=image.scaledToWidth(m_ui->m_exrCutImage->width());
        m_ui->m_exrCutImage->setPixmap(QPixmap::fromImage(toFit));
      }

    }
    catch( Error& e )
    {
      std::cerr<<"Failed to write region map." <<std::endl;;
      return ;
    }
    FileOut.close();
    if(m_ui->m_ribOut->isChecked() == true) RibOut.close();
    sprintf( tmp, "Finished %d regions. (FYI, there are %d shadowed lights. Lights range from [%0.2f,%0.2f].)", (int)regions.size(), shadowCount, minLum, maxLum );
    //std::cerr<<tmp<<std::endl;
    m_ui->m_debugOutput->append(tmp);
}


//----------------------------------------------------------------------------------------------------------------------
void MainWindow::LoadEXR()
{
  QString fileName;
  // load the file by getting the file name from the dialog
  fileName = QFileDialog::getOpenFileName(this,tr("Open File"), "", "exr files (*.exr)");
  // if this is not empty
  if (!fileName.isEmpty())
      {
      m_exrName =fileName;
      std::cout<<m_exrName.toStdString()<<std::endl;
      EXR exrImage;
      bool exrLoaded=exrImage.load(fileName.toAscii());
      if(exrLoaded == true)
      {
        int Width=exrImage.width();
        int Height=exrImage.height();

        // now load the exr pixels to the byte array
        Imf::Rgba Pixel;
        //std::cout<<"Width "<<Width<<" Height "<<Height<<std::std::endl;
        QImage image(Width, Height, QImage::Format_RGB16);
        QRgb value;

        for(int y=0; y<Height; ++y)
        {
          for(int x=0; x<Width; ++x)
          {
            Pixel=exrImage.pixel(x,y);

            value =qRgb(Pixel.r*255, Pixel.g*255, Pixel.b*255);
            image.setPixel(x, y, value);
          }
        }
        m_ui->m_exrImageLabel->setScaledContents(true);
        QImage toFit=image.scaledToWidth(m_ui->m_exrImageLabel->width());
        m_ui->m_exrImageLabel->setPixmap(QPixmap::fromImage(toFit));
      }

      }


}


//----------------------------------------------------------------------------------------------------------------------
MainWindow::~MainWindow()
{
    delete m_ui;
}

//----------------------------------------------------------------------------------------------------------------------
void MainWindow::changeEvent(
                             QEvent *_e
                            )
{
    QMainWindow::changeEvent(_e);
    switch (_e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
//----------------------------------------------------------------------------------------------------------------------
