#ifndef __MEDIANCUT_H__
#define __MEDIANCUT_H__
#include <vector>
#include <iostream>
#include "util.h"
#include "point.h"
#include "Region.h"
#include <ImfRgbaFile.h>
#include <ImfStringAttribute.h>
#include <ImfMatrixAttribute.h>
#include <ImfArray.h>

class MCRegion;
// The MedianCut object


class MedianCut
{
public:


	// Load the median cut image
  MedianCut(
            const char * _img,
            bool& _result
           );
	// Delete the sums array
	~MedianCut();

	// Get the luminance for a colour
  static float Lum(
                   const Imf::Rgba& _c
                  );

	// Generate a spherical coordinate from an image point
  static void	GetSphericalCoordinateDome(
                                         float _x,
                                         float _y,
                                         float& o_theta,
                                         float& o_phi
                                        );

  static void	GetSphericalCoordinateSphere(
                                           float _x,
                                           float _y,
                                           float& o_theta,
                                           float& o_phi
                                          );

	// Get the median
  static int Median(
                    const float * _sums,
                    int _count
                   );

	// Cut the image and generate the light positions
  int		cut(
            int _n,
            std::vector<MCRegion>& o_regions
           );

	// Create the light sources
  void 	createLightsFromRegions(
                                const std::vector<MCRegion>& _regions
                               );

	// Write a region map
  bool 	writeRegionMap(
                        const char * _filename,
                        const std::vector<MCRegion>& _regions
                      ) const;

	// Get a light
  const Point& getLight(
                        int _index
                       ) const;

	// Get the sum of a region
  Imf::Rgba sumRegion(
                       const MCRegion& _r
                     ) const;

private:

	// The EXR file
	EXR		m_exr;
	// The light list
	std::vector<Point>	m_lights;
	// The sums array
	float * m_sums;
};



#endif
