#include "exr.h"
#include "MedianCut.h"

//----------------------------------------------------------------------------------------------------------------------
// Destructor
EXR::~EXR()
{
  if ( m_pixels )
  {
    delete [] m_pixels;
  }
  if ( m_lums )
  {
    delete [] m_lums;
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Load an EXR
bool EXR::load(
               const char *_fileName
               )
{
	// Open the file
  Imf::RgbaInputFile file( _fileName );

	// Get the dimensions
	Imath::Box2i dw = file.dataWindow();
	m_width = dw.max.x - dw.min.x + 1;
	m_height = dw.max.y - dw.min.y + 1;

	// Delete the old buffers
  if ( m_pixels )
  {
    delete [] m_pixels;

  }
  if ( m_lums )
  {
    delete [] m_lums;
  }
	// Create the buffers
	m_pixels = new Imf::Rgba[ m_width * m_height ];
	m_lums = new float[ m_width * m_height ];
	if ( !m_pixels || !m_lums )
	{
    std::cerr<<"Couldn't allocate memory for images" <<std::endl;
		return false;
	}

	// Reset the image
	file.setFrameBuffer( m_pixels, 1, m_width);

	// Read the pixels
	file.readPixels( dw.min.y, dw.max.y );

	// Copy to the luminance map
	for ( int i = 0; i < m_height * m_width; ++i )
	{
		// Copy the luminance
		m_lums[i] = MedianCut::Lum( m_pixels[i] );
	}


	char tmp[256];
  sprintf( tmp, "Loaded %s (%dx%d)", _fileName, m_width, m_height );
  std::cerr<< tmp <<std::endl;

	return true;
}

//----------------------------------------------------------------------------------------------------------------------
// Return the dimensions
int EXR::width() const { return m_width; }
//----------------------------------------------------------------------------------------------------------------------
int EXR::height() const { return m_height; }


//----------------------------------------------------------------------------------------------------------------------
// Get a coloured pixel
Imf::Rgba EXR::pixel(
                      const int _x,
                      const int _y
                    )const
{
  if ( _x >= m_width )
		throw Error( "x index out of range for EXR::pixel()" );
  if ( _y >= m_height )
		throw Error( "y index out of range for EXR::pixel()" );

	// Get the pixel
  Imf::Rgba c( m_pixels[ _y * m_width + _x ] );

	// Return it
	return c;
}

//----------------------------------------------------------------------------------------------------------------------
// Get a luminance pixel
float EXR::lum(
               const int _x,
               const int _y
              ) const
{
  if ( _x >= m_width )
  {
    throw Error( "x index out of range for EXR::lum()" );
  }
  if ( _y >= m_height )
  {
    throw Error( "y index out of range for EXR::lum()" );
  }
	// Get the luminance
  float f = m_lums[ _y * m_width + _x ];

	// Return it
	return f;
}
//----------------------------------------------------------------------------------------------------------------------
